"""
日期:2023-12-01
内容:
- 集成钉钉和企业微信的webhook能力
参考文档:
- https://open.dingtalk.com/document/robots/custom-robot-access
- https://developer.work.weixin.qq.com/document/path/91770
"""
from base64 import b64encode
from enum import Enum
from hashlib import sha256
from hmac import new as hmac_new
from requests import session
from time import time
from urllib.parse import quote_plus

class WebHookType(int,Enum):
    DINGTALK=0
    WECOM=1


class WebHook:
    def __init__(self,webhook:WebHookType,url:str,sk:str = None) -> None:
        """Webhook对象

        Args:
            webhook:WebHookType,webhook的类型
            url:str,webhook的url
            sk:str,webhook的密钥,由webhook提供
        """
        self.__webhook = webhook
        self.__url = url
        self.__sk = sk

    @property
    def webhook(self):
        return self.__webhook

    @property
    def url(self):
        return self.__url
    
    @property
    def sk(self):
        return self.__sk

    def text(self,content,phone:list[str]=None,id:list[str]=None,all:bool=False)->dict:
        """text类型,支持钉钉和企微

        Args:
            content:str,发送内容
            phone:list[str],需要@的用户手机号,比如['151****1234', '151****5678']
            id:list[str],需要@的用户ID,比如['id001', 'id002']
            all:bool,是否@ALL, 仅钉钉有效,企微请在id或phone里用'@all'来表示
        """
        result = dict(msgtype='text')
        text=dict(content=content)

        match(self.webhook):
            case WebHookType.DINGTALK:
                at = dict(atMobiles=phone,isAtAll=all,atUserIds=id)
                result['at'] = at
            case WebHookType.WECOM:
                text['mentioned_list']=id
                text['mentioned_mobile_list']=phone
        result['text']=text
        return result

    def link(self,title:str,text:str,url:str,pic:str='')->dict:
        """link类型,仅支持钉钉

        Args:
            title:str,消息标题
            text:str,消息内容.如果太长只会展示部分
            url:str,点击消息跳转URL
            pic:str,图片URL
        """
        result = dict(msgtype='link')
        link = dict(title=title,text=text,messageUrl=url,picUrl=pic)
        result['link']=link
        return result

    def markdown(self,text:str,title:str='')->dict:
        """markdown类型,支持钉钉和企微

        Args:
            text:str,markdown内容
            title:str,markdown标题,仅钉钉需要填写
        """
        result = dict(msgtype='markdown')

        match(self.webhook):
            case WebHookType.DINGTALK:
                markdown=dict(text=text,title=title)
            case WebHookType.WECOM:
                markdown=dict(content=text)
        
        result['markdown']=markdown
        return result

    def send(self,data:dict)->None:
        """发送webhook消息

        Args:
            data:dict,Webhook消息,可用类的text,markdown等方法构造
        """
        url = self.url
        sk = self.sk

        match(self.webhook):
            case WebHookType.DINGTALK:
                # 钉钉的处理
                if sk:
                    sk_enc = sk.encode('utf-8')
                    timestamp = str(round(time()*1000))
                    sign_str = f'{timestamp}\n{sk}'
                    sign_str_enc = sign_str.encode('utf-8')
                    hmac_code = hmac_new(sk_enc, sign_str_enc,
                                 digestmod=sha256).digest()
                    sign = quote_plus(b64encode(hmac_code))
                    url = f'{url}&timestamp={timestamp}&sign={sign}'

        # 消息发送
        with session() as s:
            result = s.post(url,json=data)

        rsp = result.json()
        if rsp['errcode'] != 0:
            raise Exception(rsp['errmsg'])

if __name__ == '__main__':
    dt = WebHook(WebHookType.DINGTALK,'url','sk')

    data = dt.text('钉钉测试内容',phone=['151****9832'])
    data = dt.link(title='百度',text='点我可以打开baidu的地址',url='https://www.baidu.com')
    data = dt.markdown('# 这是markdown的内容',title='markdown文本')
    dt.send(data)


    wb =  WebHook(WebHookType.WECOM,'url')
    data = wb.text('微信测试内容',phone=['@all'])
    data = wb.markdown('# 这是markdown的内容')
    wb.send(data)



"""
作者:J.Hu
日期:2023-12-19
内容:
# 功能描述：
 - 支持邮件发送
# 重要版本变更说明：
 - 更新附件的代码编写,使其支持正常的文字显示;
 - To/Cc/Bcc的列表拼接采用逗号的形式拼接,原因是原来采用多个To/Cc/Bcc会导致
   在某些邮件客户端上只显示一个To/Cc/Bcc内容.
   比如,Header里面内容如:
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样只会显示一个huj,
   现在修改变成：
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>,=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样就可以显示出2个huj,huj了
"""
from smtplib import SMTP, SMTP_SSL
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
from email.header import Header


class EmailSender:
    def __init__(self,host:str,port:int=465,ssl:bool=True,timeout:int=10,encode:str='utf-8',mail_type:str='html',attach_type:str='base64')->None:
        """Email对象

        Args:
            host:str,STMP的服务器地址
            port:int,STMP的端口号,默认465
            ssl:bool,是否使用ssl,默认true
            timeout:int,服务器连接超时(秒)
            encode:str,邮件编码,默认为utf-8
            mail_type:str,邮件正文类型,默认为html
            attch_type:str,邮件附件类型,默认为base64
        """
        # 根据SSL确定SMTP的类型
        smtpfunc = SMTP_SSL if ssl else SMTP
        self.__smtp = smtpfunc(host,port,timeout=timeout)

        self.encode=encode
        self.mail_type=mail_type
        self.attach_type=attach_type

        # 初始化to,cc,bcc对象

    @property
    def smtp(self):
        return self.__smtp
    

    @property
    def sender(self):
        return self.__sender
    
    def login(self,acct:str,passwd:str,nickname:str=None) ->None:
        """登录SMTP服务器

        Args:
            acct:str,账号,比如:joestarhu@163.com
            passwd:str,密码
            nickname:str,昵称
        """
        self.smtp.login(acct,passwd)
        if not nickname:
            nickname = acct
        self.__sender = formataddr([nickname,acct],self.encode)
        self.acct = acct
        

    def send(self,title:str,content:str,attach_list:list[str]=[],to:list[str]=None,cc:list[str]=None,bcc:list[str]=None)->None:
        """发送邮件

        Args:
            title:str,邮件正文标题
            content:str,邮件正文内容
            attach_list:list,邮件附件列表, ['/user/a.zip','/user/b.zip']
            to:list,收件人列表,['a@a.com','b@b.com']
            cc:list,cc列表,['a@a.com','b@b.com']
            bcc:list,bcc列表,['a@a.com','b@b.com']
        """
        mime = MIMEMultipart()

        # 设置邮件标题和发件人
        mime['Subject'] = Header(title,self.encode)
        mime['From'] = self.sender

        # 设置收件人信息 to,cc,bcc
        receive_lst = []
        rcpt_to = dict(label='To',value=to)
        rcpt_cc = dict(label='Cc',value=cc)
        rcpt_bcc = dict(label='Bcc',value=bcc)
        
        for val in [rcpt_to,rcpt_cc,rcpt_bcc]:
            # 没有设置相关的收件人信息
            rcpt_type = val['label']
            rcpt_value = val['value']

            if not rcpt_value:
                continue
            receive_lst.extend(rcpt_value)
            mime[rcpt_type] = ','.join([formataddr([None, mail],self.encode) for mail in rcpt_value])

        # 正文内容设置
        mime.attach(MIMEText(content, self.mail_type, self.encode))

        # 附件内容设置
        for idx, val in enumerate(attach_list):
            att = MIMEText(open(val,'rb').read(),self.mail_type,self.encode)
            att.add_header('Content-ID', str(idx))
            att.add_header('Content-Type', 'application/octet-stream')
            att_name = val.split('/')[-1]
            att.add_header("Content-Disposition", 'attachment',filename=(self.encode, '', att_name))
            mime.attach(att)

        # 发送邮件
        self.smtp.sendmail(self.acct,receive_lst,mime.as_string())

    
    def close(self):
        """关闭smtp链接
        """     
        self.smtp.close()



if __name__ == '__main__':
    host='host'
    e = EmailSender(host)
    acct = 'a@a.com'
    passwd='passwd'
    nickname='joestarhu'
    e.login(acct,passwd,nickname)
    e.send(title='title',content='content',to=['a@a.com','b@b.com'])
